#ifndef GRDEFS_H
#define GRDEFS_H

/* Begin Status/Return value declarations */

enum {
	GR_ALLOW = 0x01,
	GR_DENY = 0x02,
	GR_NOTFOUND = 0x04
};

/* Begin GrSecurity status declarations */

enum {
	GR_READY = 0x01,
	GR_LEARNING = 0x02,
	GR_STATUS_INIT = 0x00 // disabled state
};

/* Table Size Defaults (in the form of 2^GR_TABLE_SIZE). Since
 * our tables are dynamic, this is just the starting point, set to
 * a reasonable value for the predicted number of MAC labels */
enum {
	GR_TABLE_SIZE = PAGE_SIZE	/* One page by default */
};

/* Begin  ACL declarations */

/* ACL Subject and Object mode flags */
enum {
	GR_FIND = 0x10,
	GR_EXISTS = 0x20,	/* ACL exists....since our hash table 
				   will have blank entries */
	GR_INHERIT = 0x40,
	GR_DELETED = 0x80
};

/* ACL Object-only mode flags */
enum {
	GR_READ = 0x01,
	GR_APPEND = 0x02,
	GR_WRITE = 0x04,
	GR_EXEC = 0x08
};

/* ACL subject-only mode flags */
enum {
	GR_KILL = 0x01,
	GR_VIEW = 0x02,
	GR_INHERIT_ONCE = 0x04,
	GR_INHERIT_NONE = 0x08,
	GR_PROTECTED = 0x100
};

/* To be implemented : ID based rulesets */

#if 0
enum {
	GR_UID = 0x01,
	GR_GID = 0x02,
	/* for ID acls(to distinguish between UID and GID) */
	GR_ALL = 0x04,
	/* for the 1 UID or GID * ACL the user is allowed to set */
	GR_ID_TPE = 0x01,
	GR_ID_FORKBOMB = 0x02,
	GR_ID_PTRACE = 0x04,
	GR_ID_EXECLOG = 0x08
};

enum {
	/* For ID acls using ports. Specifies whether a port 
	 * is for binding or connecting */
	GR_SOCKET_BIND = 0x01,
	GR_SOCKET_CONNECT = 0x02
};
#endif

/* Name hash table declarations. Used when mapping filename->inode/dev
 * and to tell whether a particular inode/dev/filename triplet has 
 * a process subject, process label, and/or file label assigned to it */
enum {
	GR_NAME_ACL_SUBJECT = 0x01,
	GR_NAME_ACL_OBJECT = 0x02,
};

/* If we have an acl for a file that did not exist at initialization,
 * set the inode/dev pair to something that will not conflict with
 * anything else (since device 0 is the NULL device we're fine :) 
 * *NOTE* - To be Implemented */
enum {
	GR_NOFILE_INODE = 42,
	GR_NOFILE_DEVICE = 0
};

#define PW_CONF		"/etc/grsec/pw"

enum {
	SET_INHERIT = 1,
	SET_PROC = 2
};

enum {
	ACL_SUBJ_LABEL_OFFSET = 1
/* The first label is godmode, thus all process subject labels must begin 
 * at offset 1 */
};

/* Begin hash section */

#define GR_SHA1HANDSOFF		/* Copies data before messing with it. */

struct gr_SHA1_CTX {
	unsigned long state[5];
	unsigned long count[2];
	unsigned char buffer[64];
};

#define rol(value, bits) (((value) << (bits)) | ((value) >> (32 - (bits))))

/* blk0() and blk() perform the initial expand. */
/* I got the idea of expanding during the round function from SSLeay */
#ifdef __LITTLE_ENDIAN
#define blk0(i) (block->l[i] = (rol(block->l[i],24)&0xFF00FF00) \
		|(rol(block->l[i],8)&0x00FF00FF))
#else
#define blk0(i) block->l[i]
#endif				/* __LITTLE_ENDIAN */
#define blk(i) (block->l[i&15] = rol(block->l[(i+13)&15]^block->l[(i+8)&15] \
			^block->l[(i+2)&15]^block->l[i&15],1))

	/* (R0+R1), R2, R3, R4 are the different operations used in SHA1 */
#define R0(v,w,x,y,z,i) z+=((w&(x^y))^y)+blk0(i)+0x5A827999+rol(v,5);w=rol(w,30);
#define R1(v,w,x,y,z,i) z+=((w&(x^y))^y)+blk(i)+0x5A827999+rol(v,5);w=rol(w,30);
#define R2(v,w,x,y,z,i) z+=(w^x^y)+blk(i)+0x6ED9EBA1+rol(v,5);w=rol(w,30);
#define R3(v,w,x,y,z,i) z+=(((w|x)&y)|(w&x))+blk(i)+0x8F1BBCDC+rol(v,5);w=rol(w,30);
#define R4(v,w,x,y,z,i) z+=(w^x^y)+blk(i)+0xCA62C1D6+rol(v,5);w=rol(w,30);

/* End hash section */

#endif
