/* * * * * * * * * * * * * * * * * * * *
 * Main grsecurity header file
 * * * * * * * * * * * * * * * * * * * */

/* Because this header file is used by the other sections of the kernel
 * it defines external functions. The actual code in kernel/gracl.c
 * uses include/linux/gracl.h. We make sure that that code ignores our
 * extern declarations to avoid conflicts */

#ifndef GR_ACL_H
#include <linux/dcache.h>
#include <linux/grdefs.h>

extern int gr_disable;
extern int gr_acl_is_enabled(void);
extern int gr_check_hidden_file(struct nameidata *nd);
extern int gr_search_file(struct dentry *dentry, __u16 mode,
			  struct vfsmount *mnt);
extern int gr_check_root(struct nameidata * nd, __u16 mode);
extern int gr_search_id(uid_t uid, gid_t gid, __u8 mode);
extern int gr_search_socket(uid_t uid, gid_t gid, __u8 mode, unsigned int port);
extern int gr_set_proc_label(struct dentry *dentry, struct vfsmount *mnt,
			     const char *filename);
extern int gr_check_hidden_task(struct task_struct *tsk);
extern int gr_check_protected_task(struct task_struct *tsk);
extern void gr_copy_label(struct task_struct *tsk);
extern void gr_handle_delete(const ino_t ino, const kdev_t dev);
extern void gr_handle_create(const char *filename, const struct nameidata *nd);
extern void gr_set_caps(void);
extern int gr_check_create(struct dentry *new_dentry, struct dentry *parent,
			   struct vfsmount *mnt, __u16 mode);
#endif

#ifndef GR_SECURITY_H
#define GR_SECURITY_H
#include <linux/binfmts.h>

extern spinlock_t task_capability_lock;

extern __u16 ip_randomid(void);

extern int grsec_enable_link;
extern int grsec_enable_fifo;
extern int grsec_enable_execve;
extern int grsec_enable_capprot;
extern int grsec_enable_forkbomb;
extern int grsec_forkbomb_gid;
extern int grsec_forkbomb_sec;
extern int grsec_forkbomb_max;
extern int grsec_enable_execlog;
extern int grsec_enable_signal;
extern int grsec_enable_forkfail;
extern int grsec_enable_time;
extern int grsec_enable_dmesg;
extern int grsec_enable_chroot_sig;
extern int grsec_enable_chroot_findtask;
extern int grsec_enable_chroot_mount;
extern int grsec_enable_chroot_double;
extern int grsec_enable_chroot_pivot;
extern int grsec_enable_chroot_chdir;
extern int grsec_enable_chroot_chmod;
extern int grsec_enable_chroot_mknod;
extern int grsec_enable_chroot_fchdir;
extern int grsec_enable_chroot_ptrace;
extern int grsec_enable_chroot_nice;
extern int grsec_enable_chroot_execlog;
extern int grsec_enable_chroot_caps;
extern int grsec_enable_tpe;
extern int grsec_tpe_gid;
extern int grsec_enable_tpe_glibc;
extern int grsec_enable_tpe_all;
extern int grsec_enable_sidcaps;
extern int grsec_enable_randpid;
extern int grsec_enable_randid;
extern int grsec_enable_randsrc;
extern int grsec_enable_randrpc;
extern int grsec_enable_randbind;
extern int grsec_enable_randping;
extern int grsec_enable_socket_all;
extern int grsec_socket_all_gid;
extern int grsec_enable_socket_client;
extern int grsec_socket_client_gid;
extern int grsec_enable_socket_server;
extern int grsec_socket_server_gid;
extern int grsec_audit_gid;
extern int grsec_enable_group;
extern int grsec_enable_audit_ipc;
extern int grsec_enable_audit_ptrace;
extern int grsec_enable_mount;
extern int grsec_enable_chdir;
extern int grsec_lock;

extern const char *captab_log[29];
extern struct task_struct *child_reaper;

#define proc_is_chrooted(tsk_a)  ((tsk_a->fs->root->d_inode->i_dev != \
			  child_reaper->fs->root->d_inode->i_dev) || \
			  (tsk_a->fs->root->d_inode->i_ino != \
			  child_reaper->fs->root->d_inode->i_ino))

#define have_same_root(tsk_a,tsk_b) ((tsk_a->fs->root->d_inode->i_dev == \
			  tsk_b->fs->root->d_inode->i_dev) && \
			  (tsk_a->fs->root->d_inode->i_ino == \
			  tsk_b->fs->root->d_inode->i_ino))

#define DEFAULTSECMSG "(%.16s:%d) UID(%d) EUID(%d), parent (%.16s:%d) " \
		      "UID(%d) EUID(%d)"

#define DEFAULTSECARGS current->comm, current->pid, current->uid, \
		       current->euid, current->p_pptr->comm, \
		       current->p_pptr->pid, current->p_pptr->uid, \
		       current->p_pptr->euid

#define CHROOTLONGCHECK t->pid && t->fs && t->fs->root && \
			t->fs->root->d_inode && t->p_pptr && \
              		current->pid && current->fs && current->fs->root && \
			current->fs->root->d_inode && \
              		current->p_pptr && child_reaper && child_reaper->pid && \
			child_reaper->fs && child_reaper->fs->root && \
			child_reaper->fs->root->d_inode && \
			grsec_enable_chroot_sig

#define GR_CHROOT_CAPS ( \
	CAP_TO_MASK(CAP_FOWNER) | CAP_TO_MASK(CAP_SYS_RESOURCE) | \
	CAP_TO_MASK(CAP_LINUX_IMMUTABLE) | CAP_TO_MASK(CAP_NET_ADMIN) | \
	CAP_TO_MASK(CAP_SYS_MODULE) | CAP_TO_MASK(CAP_SYS_RAWIO) | \
	CAP_TO_MASK(CAP_SYS_PACCT) | CAP_TO_MASK(CAP_SYS_ADMIN) | \
	CAP_TO_MASK(CAP_SYS_BOOT) | CAP_TO_MASK(CAP_SYS_TIME) | \
	CAP_TO_MASK(CAP_NET_RAW) | CAP_TO_MASK(CAP_SYS_TTY_CONFIG))


void gr_handle_exec_args(struct linux_binprm * bprm, char ** argv);
int gr_tpe_allow(struct file * file);
int gr_tpe_all_allow(struct file * file);
int gr_tpe_bad_env(char ** envpp, struct linux_binprm * bprm);
int gr_acl_bad_env(char ** envpp, const char * filename);

#endif
		

